@@ -17,12 +17,11 @@ import os
 from types import SimpleNamespace
 
 from conftest import KFP_COMPONENT_CACHE_INSTANCE
+from conftest import TEST_CATALOG_NAME
 import jupyter_core.paths
 import pytest
 
-from elyra.metadata.manager import MetadataManager
 from elyra.metadata.metadata import Metadata
-from elyra.metadata.schemaspaces import ComponentCatalogs
 from elyra.pipeline.catalog_connector import FilesystemComponentCatalogConnector
 from elyra.pipeline.catalog_connector import UrlComponentCatalogConnector
 from elyra.pipeline.component_catalog import ComponentCache
@@ -47,7 +46,7 @@ def test_component_catalog_can_load_components_from_registries(component_cache_i
     assert len(components) > 0
 
 
-def test_modify_component_catalogs():
+def test_modify_component_catalogs(metadata_manager_with_teardown):
     # Initialize a ComponentCache instance and wait for all worker threads to compete
     component_catalog = ComponentCache.instance()
     component_catalog.wait_for_all_cache_updates()
@@ -58,8 +57,6 @@ def test_modify_component_catalogs():
     # Components must be sorted by id for the equality comparison with later component lists
     initial_components = sorted(initial_components, key=lambda component: component.id)
 
-    metadata_manager = MetadataManager(schemaspace=ComponentCatalogs.COMPONENT_CATALOGS_SCHEMASPACE_ID)
-
     # Create new registry instance with a single URL-based component
     paths = [_get_resource_path('kfp_test_operator.yaml')]
 
@@ -70,17 +67,11 @@ def test_modify_component_catalogs():
         "paths": paths
     }
     registry_instance = Metadata(schema_name="local-file-catalog",
-                                 name="new_test_registry",
+                                 name=TEST_CATALOG_NAME,
                                  display_name="New Test Registry",
                                  metadata=instance_metadata)
 
-    try:
-        if metadata_manager.get("new_test_registry"):
-            metadata_manager.remove("new_test_registry")
-    except Exception:
-        pass
-
-    metadata_manager.create("new_test_registry", registry_instance)
+    metadata_manager_with_teardown.create(TEST_CATALOG_NAME, registry_instance)
 
     # Wait for update to complete
     component_catalog.wait_for_all_cache_updates()
@@ -96,7 +87,7 @@ def test_modify_component_catalogs():
 
     # Modify the test registry to add an additional path to
     paths.append(_get_resource_path('kfp_test_operator_no_inputs.yaml'))
-    metadata_manager.update("new_test_registry", registry_instance)
+    metadata_manager_with_teardown.update(TEST_CATALOG_NAME, registry_instance)
 
     # Wait for update to complete
     component_catalog.wait_for_all_cache_updates()
@@ -111,7 +102,7 @@ def test_modify_component_catalogs():
     assert 'Test Operator No Inputs' in modified_component_names
 
     # Delete the test registry
-    metadata_manager.remove("new_test_registry")
+    metadata_manager_with_teardown.remove(TEST_CATALOG_NAME)
 
     # Wait for update to complete
     component_catalog.wait_for_all_cache_updates()
@@ -131,7 +122,7 @@ def test_modify_component_catalogs():
     assert initial_palette == post_delete_palette
 
 
-def test_directory_based_component_catalog():
+def test_directory_based_component_catalog(metadata_manager_with_teardown):
     # Initialize a ComponentCache instance and wait for all worker threads to compete
     component_catalog = ComponentCache.instance()
     component_catalog.wait_for_all_cache_updates()
@@ -139,8 +130,6 @@ def test_directory_based_component_catalog():
     # Get initial set of components from the current active registries
     initial_components = component_catalog.get_all_components(RUNTIME_PROCESSOR)
 
-    metadata_manager = MetadataManager(schemaspace=ComponentCatalogs.COMPONENT_CATALOGS_SCHEMASPACE_ID)
-
     # Create new directory-based registry instance with components in ../../test/resources/components
     registry_path = _get_resource_path('')
     instance_metadata = {
@@ -150,17 +139,11 @@ def test_directory_based_component_catalog():
         "paths": [registry_path]
     }
     registry_instance = Metadata(schema_name="local-directory-catalog",
-                                 name="new_test_registry",
+                                 name=TEST_CATALOG_NAME,
                                  display_name="New Test Registry",
                                  metadata=instance_metadata)
 
-    try:
-        if metadata_manager.get("new_test_registry"):
-            metadata_manager.remove("new_test_registry")
-    except Exception:
-        pass
-
-    metadata_manager.create("new_test_registry", registry_instance)
+    metadata_manager_with_teardown.create(TEST_CATALOG_NAME, registry_instance)
 
     # Wait for update to complete
     component_catalog.wait_for_all_cache_updates()
@@ -175,9 +158,6 @@ def test_directory_based_component_catalog():
     assert 'Test Operator' in added_component_names
     assert 'Test Operator No Inputs' in added_component_names
 
-    # Remove the test instance
-    metadata_manager.remove("new_test_registry")
-
 
 def test_parse_kfp_component_file():
     # Define the appropriate reader for a filesystem-type component definition
@@ -366,7 +346,12 @@ def test_parse_kfp_component_file_no_inputs():
     assert len(properties_json['current_parameters'].keys()) == num_common_params
     assert len(properties_json['parameters']) == num_common_params
     assert len(properties_json['uihints']['parameter_info']) == num_common_params
-    assert len(properties_json['uihints']['group_info'][0]['group_info']) == num_common_params
+
+    # Total number of groups includes one for each parameter,
+    # plus one for the output group header,
+    # plus 1 for the component_source header
+    num_groups = num_common_params + 2
+    assert len(properties_json['uihints']['group_info'][0]['group_info']) == num_groups
 
     # Ensure that template still renders the two common parameters correctly
     assert properties_json['current_parameters']['label'] == ""
